# dartlin
[![coverage report](https://gitlab.com/wolfenrain/dartlin/badges/master/coverage.svg)](https://gitlab.com/wolfenrain/dartlin/-/commits/master)
[![pipeline status](https://gitlab.com/wolfenrain/dartlin/badges/master/pipeline.svg)](https://gitlab.com/wolfenrain/dartlin/-/commits/master)  
[![plugin version](https://img.shields.io/pub/v/dartlin?label=pub)](https://pub.dev/packages/dartlin)
[![dependencies](https://img.shields.io/librariesio/release/pub/dartlin?label=dependencies)](https://gitlab.com/wolfenrain/dartlin/-/blob/master/pubspec.yaml)

## Introduction

### What it does

Dartlin is a helper library that provides readable methods with which you can write cleaner looking code. 
- It allows you to have more control over your code flow. By using the `when` method for example, to write switch-like expressions.
- It provides methods to create data sets of certain types, for example the `range` method.

### What it does not do

Dartlin is by definition not a performance optimization library, the main focus is adding readable methods that help you keep your code clean. And  readability does not always equal performance optimization. So if you are looking to optimize your code, this is not the library for you.

**Note**: The performance hit of the provided methods tends to be very minimal, but most of them do rely on [closures](https://dart.dev/guides/language/language-tour#anonymous-functions) to work.

## Common Usage

### Control flow

The Dartlin `control_flow` library provides methods for writing cleaner and more controllable code, these control flow methods can be used as a replacement for existing [control flow statements](https://dart.dev/guides/language/language-tour#control-flow-statements) or in combination with them. 

**Note**: Dart already provides a few interesting control flow operations for `List`s and `Map`s. So before you decide to use Dartlin, first read more about [collection operators](https://dart.dev/guides/language/language-tour#collection-operators). It may already suit your needs.

#### if-statement as expressions

The `iff` method works like the if statement, but with the added bonus of being able to write them as expressions. You can use them to replace complex ternary operators with a readable if-like statement:
```dart
final x = iff(a < b, () {
  return a;
}).elseIf(a == b, () {
  return 0;
}).orElse(() {
  return b;
});
```
See the [iff](https://pub.dev/documentation/dartlin/latest/control_flow/iff.html) docs for more information.

#### switch-like statements as expressions

The `when` method replaces the switch statement. And can be used to write expressions:
```dart
final result = when(place, {
  1: () => CompetitionPlace.first,
  2: () => CompetitionPlace.second,
  3: () => CompetitionPlace.third,
  [4,5]: () => CompetitionPlace.honourableMentions,
  orElse: () => CompetitionPlace.others
});
```
See the [when](https://pub.dev/documentation/dartlin/latest/control_flow/when.html) docs for more information.

# Development and Contributing
Interested in contributing? We love merge requests! See the [Contribution](https://gitlab.com/wolfenrain/dartlin/blob/master/CONTRIBUTING.md) guidelines.

### Collections

#### Creating progression ranges

The `range` method allows the user to easily create a lists of certain `num` types. While being able to define the start value, the end value, the steps it should take to reach the end value and if it should be created in reverse or not:
```dart
// [0, 1, 2, 3, 4]
final list1 = range(0, to: 4);

// [4, 3, 2, 1, 0]
final list2 = range(4, downTo: 0); 

// [1, 3, 5, 7]
final list3 = range(1, to: 8, step: 2);

// [1, 2, 3]
final list4 = range(0, until: 4);
```

See the [range](https://pub.dev/documentation/dartlin/latest/collections/range.html) docs for more information.