part of control_flow;

/// Used for checking if the [ElseIf] and [OrElse] are called from a [iff] call.
const dynamic _iff_hack = '__iff__hack__';

/// The [iff] method can be used as a normal if statement, or as an expression, i.e. it returns a value.
///
/// The [iff] method can replace complex ternary operators so your code becomes more readable:
/// ```dart
/// // Traditional usage
/// final x = a < b ? a : a == b ? 0 : b;
///
/// // iff usage
/// final x = iff(a < b, () {
///   return a;
/// }).elseIf(a == b, () {
///   return 0;
/// }).orElse(() {
///   return b;
/// });
/// ```
///
/// If you are using the [iff] method as an expression(returning its value or assigning it to a variable),
/// then the expression is required to make use of the `[OrElse] method.
T iff<T>(bool statement, T Function() branch) {
  if (statement) {
    return branch();
  }
  return _iff_hack;
}

extension ElseIf<T> on T {
  T elseIf(bool statement, T Function() branch) {
    if (this == _iff_hack) {
      if (statement) {
        return branch();
      }
      return _iff_hack;
    }
    return this;
  }
}

extension OrElse<T> on T {
  T orElse(T Function() branch) {
    if (this == _iff_hack) {
      return branch();
    }
    return this;
  }
}
